'use strict';

exports.__esModule = true;
exports.default = babelPluginTransformMetadata;

var _getTypesInfo = require('./visitors/getTypesInfo');

var _getTypesInfo2 = _interopRequireDefault(_getTypesInfo);

var _addReflections = require('./visitors/addReflections');

var _addReflections2 = _interopRequireDefault(_addReflections);

var _createTypeForAnnotation = require('./factories/createTypeForAnnotation');

var _createTypeForAnnotation2 = _interopRequireDefault(_createTypeForAnnotation);

var _createCreateCreateGenericTypeMetadata = require('./factories/createCreateCreateGenericTypeMetadata');

var _createCreateCreateGenericTypeMetadata2 = _interopRequireDefault(_createCreateCreateGenericTypeMetadata);

var _createCreateCreateObjectTypeMetadata = require('./factories/createCreateCreateObjectTypeMetadata');

var _createCreateCreateObjectTypeMetadata2 = _interopRequireDefault(_createCreateCreateObjectTypeMetadata);

var _createTypeForAnnotations = require('./factories/createTypeForAnnotations');

var _createTypeForAnnotations2 = _interopRequireDefault(_createTypeForAnnotations);

var _createGetUniqueTypeName = require('./factories/createGetUniqueTypeName');

var _createGetUniqueTypeName2 = _interopRequireDefault(_createGetUniqueTypeName);

var _createParentPathInsertAfter = require('./modifiers/createParentPathInsertAfter');

var _createParentPathInsertAfter2 = _interopRequireDefault(_createParentPathInsertAfter);

var _createReplaceMagicTypeCasts = require('./modifiers/createReplaceMagicTypeCasts');

var _createReplaceMagicTypeCasts2 = _interopRequireDefault(_createReplaceMagicTypeCasts);

var _createInjectParamTypes = require('./metaCreators/createInjectParamTypes');

var _createInjectParamTypes2 = _interopRequireDefault(_createInjectParamTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function babelPluginTransformMetadata(_ref) {
    var t = _ref.types;

    return {
        visitor: {
            Program: function Program(path, _ref2) {
                var opts = _ref2.opts;

                var getUniqueTypeName = (0, _createGetUniqueTypeName2.default)(opts.typeNameStrategy || 'typeName');
                var state = {
                    getUniqueTypeName: getUniqueTypeName,
                    reflectImport: opts.reflectImport,
                    ambiantTypeCastImport: opts.ambiantTypeCastImport || 'babel-plugin-transform-metadata/_',
                    ambiantDepsImport: opts.ambiantDepsImport || 'babel-plugin-transform-metadata/Deps',

                    lastImportPath: null,
                    reservedGenerics: new Set(opts.reservedGenerics || ['Class', 'ResultOf']),
                    injectId: null,
                    ambiantTypeCast: null,
                    externalClassNames: new Map(),
                    internalTypes: new Map(),
                    externalTypeNames: new Map(),
                    exportNames: new Map()
                };
                path.traverse(_getTypesInfo2.default, state);
                var replaceMagicTypeCasts = (0, _createReplaceMagicTypeCasts2.default)(t, state.externalTypeNames);
                var createCreateObjectTypeMetadata = (0, _createCreateCreateObjectTypeMetadata2.default)(t);
                var createCreateGenericTypeMetadata = (0, _createCreateCreateGenericTypeMetadata2.default)(t, state.externalTypeNames, state.internalTypes, state.reservedGenerics, state.externalClassNames);
                var typeForAnnotation = (0, _createTypeForAnnotation2.default)(t, state.externalTypeNames, createCreateObjectTypeMetadata, createCreateGenericTypeMetadata);
                var typeForAnnotations = (0, _createTypeForAnnotations2.default)(typeForAnnotation);

                var injectId = state.injectId;
                var injectorDeclaration = null;
                if (!injectId && opts.reflectImport) {
                    injectId = t.identifier('Driver');
                    injectorDeclaration = t.importDeclaration([t.importDefaultSpecifier(injectId)], t.stringLiteral(opts.reflectImport));
                }

                var defineParamTypes = (0, _createInjectParamTypes2.default)(t, injectId, typeForAnnotations, opts.paramKey || 'design:paramtypes', opts.typeKey || 'design:function');
                var parentPathInsertAfter = (0, _createParentPathInsertAfter2.default)(defineParamTypes);

                var reflectionState = {
                    magicTypeCasts: [],
                    parentPaths: [],
                    onlyExports: opts.onlyExports || false,
                    exportNames: state.exportNames,
                    magicTypeCastExpression: state.ambiantTypeCast ? state.ambiantTypeCast.node.specifiers[0].local.name : ''
                };

                path.traverse(_addReflections2.default, reflectionState);

                reflectionState.magicTypeCasts.forEach(replaceMagicTypeCasts);
                reflectionState.parentPaths.forEach(parentPathInsertAfter);

                if (injectorDeclaration) {
                    if (state.lastImportPath) {
                        state.lastImportPath.insertAfter(injectorDeclaration);
                    } else {
                        path.node.body.unshift(injectorDeclaration);
                    }
                }

                if (state.ambiantTypeCast) {
                    state.ambiantTypeCast.remove();
                }
            }
        }
    };
}
//# sourceMappingURL=index.js.map